﻿Type=StaticCode
Version=3.5
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@

Sub Process_Globals
	Private sql As SQL
	Public const ACTION_IN = 1, ACTION_OUT = 2 As Int
End Sub

Public Sub Init
	sql.InitializeSQLite(File.DirApp, "db.db", True)
	CreateDB
End Sub

Private Sub CreateDB
	If sql.ExecQuerySingleResult("SELECT count(*) FROM sqlite_master WHERE type='table' AND name='main'") = 0 Then
		sql.ExecNonQuery("PRAGMA journal_mode = wal") 'required to support concurrent access
		sql.ExecNonQuery($"CREATE TABLE main (
							EmployeeId INTEGER,
							Time INTEGER,
							Action INTEGER,
							PRIMARY KEY (EmployeeId, Time)
							)"$)
		sql.ExecNonQuery("CREATE INDEX index1 ON main (EmployeeId, Time)")
		DBUtils.CreateTable(sql, "Employees", CreateMap("Id": DBUtils.DB_INTEGER, _
			"Name": DBUtils.DB_TEXT), "Id")
		FillEmployees
	End If
End Sub

Private Sub FillEmployees
	Dim employees As List
	employees.Initialize
	For i = 1 To 100
		employees.Add(CreateMap("Id": i, "Name": "John Doe" & i))
	Next
	DBUtils.InsertMaps(sql, "Employees", employees)
End Sub

Public Sub CheckPerson(EmployeeId As Int, Action As Int, Time As Long) 
	DBUtils.InsertMaps(sql, "main", Array(CreateMap("EmployeeId": EmployeeId, _
		"Time": Time, "Action": Action)))
End Sub

'Returns an array with the last action and last time.
Public Sub GetStatus (EmployeeId As Int) As Object()
	'get the last time
	Dim lastTime As Object = sql.ExecQuerySingleResult2("SELECT max(time) FROM main WHERE EmployeeId = ?", Array (EmployeeId))
	If lastTime = Null Then Return Array(ACTION_OUT, 0) 'never checked before
	Dim action As Int = sql.ExecQuerySingleResult2($"SELECT action FROM main WHERE 
		EmployeeId = ? AND Time = ?"$ _
		, Array (EmployeeId, lastTime))
	Return Array (action, lastTime)
End Sub

public Sub GetReportForEmployee(StartDate As Long, EndDate As Long, Id As Int) As String
	Dim rows As List = DBUtils.ExecuteMemoryTable(sql, $"SELECT time, action FROM main
										WHERE EmployeeId = ? AND time >= ? AND time <= ?"$, _
										Array As String(Id, StartDate, EndDate), 0)
	Dim lastCheckInTime As Long
	If rows.Size > 0 Then lastCheckInTime = StartDate
	Dim totalMillis As Long = 0
	Dim sb As StringBuilder
	sb.Initialize
	Dim name As String = sql.ExecQuerySingleResult2("SELECT name FROM employees WHERE id = ?", Array (Id))
	sb.Append($"*** EmployeeId: ${Id}, Name: ${name} ***
Report from $DateTime{StartDate} to $DateTime{EndDate}"$).Append(CRLF).Append(CRLF)
	For Each row() As String In rows
		Dim action As Int = row(1)
		Dim time As Long = row(0)
		If action = ACTION_OUT Then
			If lastCheckInTime = 0 Then
				sb.Append($"Incorrect data. Id: ${Id}, Time: ${time}"$).Append(CRLF)
			Else
				sb.Append($"In: $DateTime{lastCheckInTime} Out: $DateTime{time}"$).Append(CRLF)
				totalMillis = totalMillis + time - lastCheckInTime
			End If
			lastCheckInTime = 0
		Else
			lastCheckInTime = time
		End If
	Next
	If lastCheckInTime > 0 Then
			sb.Append($"In: $DateTime{lastCheckInTime} Out: $DateTime{EndDate}"$).Append(CRLF)
		totalMillis = totalMillis + EndDate - lastCheckInTime
	End If
	sb.Append(CRLF).Append($"*** Total: ${TicksToString(totalMillis)} ***"$).Append(CRLF)
	Return sb.ToString
End Sub

Private Sub TicksToString(ticks As Long) As String
	Return $"$2.0{Floor(ticks / DateTime.TicksPerHour)}:$2.0{(Floor(ticks / DateTime.TicksPerMinute) mod 60)}"$ _ 
		 & $":$2.0{Floor((ticks / DateTime.TicksPerSecond) mod 60)}"$
End Sub